<%
'   Bluestem - University of Illinois WWW Identification Service
'
'   bluestem.inc  - Bluestem Active Server Pages (ASP) VBScript API
'
'   Copyright 2010 - Board of Trustees of the University of Illinois
'
'   This module based on original work by Bill Reynen, U of I Foundation
'   Further adapted from version from UIUC Cites.
'
'   Johnathan Kupferer - jtk@uic.edu
'   ACCC
'   University of Illinois at Chicago

'----------------------------------------------------------------------
'   BluestemLogin/ReLogin - authenticate client via lb_login front-door CGI
'----------------------------------------------------------------------


Dim BluestemCgiDir, BluestemCache, BluestemAspCache, BluestemCookieDelimPort
BluestemCgiDir = "%BluestemCgiDir%"
BluestemCache = "%BluestemCache%"
BluestemAspCache = "%BluestemAspCache%"
BluestemCookieDelimPort = "~"

Public Sub BluestemLogin()
    Call BluestemLogin1(BluestemCgiDir & "/lb_login.pl")
End Sub

Public Sub BluestemLoginPrior(prior)
    Call BluestemLogin1Prior(BluestemCgiDir & "/lb_login.pl", prior)
End Sub

Public Sub BluestemReLogin()
    Call BluestemLogin1(BluestemCgiDir & "/lb_relogin.pl")
End Sub

Public Sub BluestemReLoginPrior(prior)
    Call BluestemLogin1Prior(BluestemCgiDir & "/lb_relogin.pl", prior)
End Sub

Private Sub BluestemLogin1(ByVal url)
    Call BluestemLogin1Prior(url, -1)
End Sub

Private Sub BluestemLogin1Prior(ByVal url, prior)
    Dim query
    url = url & Request.ServerVariables("SCRIPT_NAME")
    query = StripPriorQuery()
    
    ' new section for prior
    If prior > -1 Then
        ' add lb_prior=<prior> here
        If len(query) > 0 Then
            query = query + "&" & "lb_prior=" & prior
        Else
            query = "lb_prior=" & prior
        End If
    End If    
    ' End new section
    
    If len(query) > 0 Then
        url = url & "?" & query
    End If
    Response.Redirect(url)
End Sub

Private Function StripPriorQuery()
    Dim query
    query = Request.ServerVariables("QUERY_STRING")
    If (0 = Len(query)) Then
        StripPriorQuery = ""
    Else
        Dim parts
        parts = Split(query, "&")
        StripPriorQuery = query
        Dim rx
        Set rx = New RegExp
        rx.Pattern = "^lb_prior(?:=.*)?$"
        Dim k
        Dim outp
        outp = ""
        For k = LBound(parts) To UBound(parts)
            parts(k) = rx.Replace(parts(k), "")
            If (Len(parts(k)) > 0) Then
                If (Len(outp) > 0 ) Then
                    outp = outp & "&"
                End If
                outp = outp + parts(k)
            End If
        Next
        StripPriorQuery = outp
    End If
End Function


'----------------------------------------------------------------------
'   BluestemId - check if client already authenticated via Bluestem
'----------------------------------------------------------------------
Public Function BluestemId
    Dim cookiename, port, cookie, c, cacheid, clientrandom, msg
    Dim cachefile, fs, f, l, bid, user, domain, authmethod
    Dim shortid, idletime, sessiontime
    Const ForReading = 1
    Const RandomSize = 16
    Const IdChars = _
        "abcdefghijklmnopqrstuvwxyz@./-0123456789ABCDEFGHIJKLMNOPQRSTUVQXYZ"
        
    '---form Bluestem cookie name for this server----------------------
    cookiename = "Bluestem_" & Request.ServerVariables("SERVER_NAME")
    port = Request.ServerVariables("SERVER_PORT")
    If (port <> 443) Then
        cookiename = BluestemCookieDelimPort & port
    End If
        
    '---read Bluestem cookie - report login needed if none-------------
    cookie = Request.Cookies(cookiename)
    If cookie = "" Then
        BluestemId = Array("","","")
        Exit Function
    End If
        
    '---validate client cookie----------------------------------------
    c = Split(cookie,"~",-1)
    cacheid = c(0)
    clientrandom = c(1)
    If UBound(c) <> 1                                 _
        Or NotMatch(cacheid,"0123456789-")            _
        Or NotMatch(clientrandom,"0123456789abcdef")  _
        Or Len(clientrandom) <> RandomSize            _
    Then
        msg = "906 invalid cookie(" & cookie & ")"
        Call BluestemLog(msg)
        BluestemId = Array("","","")
        Exit Function
    End If

    '---Determine cachefile name from cookie cacheid------------------
    cachefile = BluestemCache & "\" & cacheid
        
    '---read cache file-----------------------------------------------
    On Error Resume Next
    Set fs = CreateObject("Scripting.FileSystemObject")
    Set f = fs.OpenTextFile(cachefile, ForReading, False)
    If Err.Number <> 0 Then
        BluestemId = Array("","","")
        Exit Function
    End If
    l = f.ReadLine
    f.close

    '---validate cache file-------------------------------------------
    c = Split(l,",",-1)
    id = c(0)
    ip = c(1)
    clientrandom2 = c(2)
    If UBound(c) < 2                                   _
        Or NotMatch(id,IdChars)                        _
        Or NotMatch(ip,"0123456789.")                  _
        Or Len(ip) > 15                                _
        Or NotMatch(clientrandom2,"0123456789abcdef")  _
        Or Len(clientrandom2) <> RandomSize            _        
    Then
        msg = "901 " & cacheid & ": invalid cache file contents(" & l & ")"
        Call BluestemLog(msg)
        BluestemId = Array("","","")
        Exit Function
    End If
        
    '---verify cookie ClientRandom matches value in cache---------------
    If clientrandom <> clientrandom2 Then
        msg = "904 " & cacheid & ": clientrandom mismatch: "  _
            & "was(" & clientrandom2 & ") now(" & clientrandom & ")"
        Call BluestemLog(msg)
        BluestemId = Array("","","")
        Exit Function
    End If

    '---ignore incomplete login-----------------------------------------
    If Mid(id,1,8) = "Unknown@" Then
        msg = "incomplete login ignored( " & cacheid & ")"
        Call BluestemLog(msg)
        BluestemId = Array("","","")
        Exit Function
    End If

    '---determine login and last active times------------------------------
    timefile = BluestemAspCache & "\" & cacheid
    Set fs = CreateObject("Scripting.FileSystemObject")
    Set f = fs.OpenTextFile(timefile, ForReading, False)

    If Err.Number = 0 Then
        logintime = f.ReadLine
        lastactive = f.ReadLine
        f.close
    Else
        logintime = Now()
        lastactive = Now()
        Err.Clear
    End If

    '---update last active time----------------------------------------
    Set fs = CreateObject("Scripting.FileSystemObject")
    Set f = fs.CreateTextFile(timefile, "True")
    f.WriteLine(logintime)
    f.WriteLine(Now())
    f.close

    '---compute time since login and last visit------------------------
    sessiontime = DateDiff("s",logintime,Now())
    idletime = DateDiff("s",lastactive,Now())

    '---return ID and times to caller----------------------------------
    BluestemId = Array(BluestemIDShort(id), idletime, sessiontime)
End Function


'----------------------------------------------------------------------
'   BluestemIdParse - extract user,domain,authmethod components of ID
'----------------------------------------------------------------------
Public Function BluestemIdParse(ByVal id)
    Dim user, domain, auth, s, t

    s = InStr(1, id, "/")
    If s > 0 Then
        auth = Mid(id, s+1)
        id = Mid (id, 1, s-1)
    End If
 
    t = split(id, "@")
    If UBound(t) > 0 Then
        user = t(0)
        domain = t(1)
    Else
        user = id
    End If
        
    If domain = "" Then
        domain = "uic.edu"
    End If
        
    If auth = "" Then
        If domain = "uic.edu" Then
            auth = "tacacs"
        Else
            auth = "default"
        End If
    End If
           
    BluestemIdParse = Array(user, domain, auth)  
End Function


'----------------------------------------------------------------------
'   BluestemIdShort - remove default domain,authmethod components of ID
'----------------------------------------------------------------------
Public Function BluestemIdShort(ByVal id)
    Dim s, short
        
    s = BluestemIdParse(id)
    short = s(0)
    If s(1) <> "uic.edu" Then
        short = short & "@" & s(1)
    End If
    If s(2) <> "tacacs" And s(2) <> "default" Then
        short = short & "/" & s(2)
    End If

    BluestemIdShort = short
End Function


'----------------------------------------------------------------------
'   BluestemIdFull  - add default domain,authmethod components of ID
'----------------------------------------------------------------------
Public Function BluestemIdFull(ByVal id)
    Dim f, full
        
    f = BluestemIdParse(id)
    BluestemIdFull = f(0) & "@" & f(1) & "/" & f(2)
End Function


'----------------------------------------------------------------------
'   BluestemLog - write Bluestem Log File
'----------------------------------------------------------------------
Public Sub BluestemLog(ByVal msg)
    Dim Days, t, D, i, S, prefix, fs, l
    Days = Array("???", "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat")
    Const ForAppending = 8

    t = Now
        
    D = Array(Year(t), Month(t), Day(t), Hour(t), Minute(t), Second(t))
    D(0) = D(0) - 1900
    D(0) = D(0) Mod 100
        
    For i = 0 To 5
        If D(i) < 10 Then
            D(i) = "0" & D(i)
        End If
    Next

    S = split(Request.ServerVariables("SCRIPT_NAME"), "/")
    prefix = D(0) & "/" & D(1) & "/" & D(2) & " "          _
           & D(3) & ":" & D(4) & ":" & D(5) & " "          _           
           & Days(Weekday(t)) & " " & S(UBound(S)) & ": "

    Set fs = CreateObject("Scripting.FileSystemObject")
    Set l = fs.OpenTextFile("c:\bluestem\logs\bluestem", ForAppending, False)
    l.WriteLine(prefix & msg)
    l.Close
End Sub


'----------------------------------------------------------------------
'   BluestemLogout - End ASP session and logout from Bluestem
'----------------------------------------------------------------------
Public Sub BluestemLogout()
    Dim logout
        
    logout = BluestemCgiDir & "/logout.pl"
    Session.Abandon
    Response.Redirect(logout)
End Sub


'----------------------------------------------------------------------
'   NotMatch - find char in string not matching one of a set of characters
'----------------------------------------------------------------------
Private Function NotMatch(ByVal string, ByVal chars)
    Dim s, c, m
                
    For s = 1 To Len(string)
        m = 0
        for c = 1 To Len(chars)
            If Mid(string,s,1) = Mid(chars,c,1) Then
                m = 1
                Exit For
            End If
        Next
        If m = 0 Then
            NotMatch = s
            Exit Function
        End If
    Next
    NotMatch = 0
End Function

%>
