#!/usr/bin/perl
#
#   Bluestem - University of Illinois WWW Identification Service
#
#   admin.cgi - Bluestem Application Server Administration
#

use strict;
use warnings;
use FileHandle ();

use lib "/usr/local/bluestem-client/lib";

eval {
	require CGI;
	require JSON::PP;
	require Bluestem::Client;
	require Bluestem::CGIClient;
};
if( $@ ) {
	print "Content-type: text/plain\n\n";
	print "Error loading libraries: $@\n";
	exit 0;
}

my($cgi,$bsc,$json) = @_;
eval {
	$cgi = CGI->new();
	$json = JSON::PP->new();
	$json->pretty(1)->canonical(1)->relaxed(1);
};
if( $@ ) {
	print $cgi->header("text/plain");
	print "Error initializing objects: $@\n";
	exit 0;
}

my $bluestem_client_path;
for my $path ( @INC ) {
	next unless -e "$path/Bluestem/Client.pm";
	$bluestem_client_path = "$path/Bluestem/Client.pm";
	last;
}

my $bsconf = $ENV{BLUESTEM_CLIENT_CONFIG} || $Bluestem::Client::BSCONF;
eval {
	FileHandle->new($bsconf,"r+")
		or die "$!\n";
};
if( $@ ) {
	print $cgi->header("text/plain");
	print "Problem opening bluestem config `$bsconf': $@\n";
	print "Webserver user should be able to read and write config.\n";
	print "Location of config is specified in $bluestem_client_path\n";
	exit 0;
}

eval {
	$bsc = Bluestem::CGIClient->new( cgi => $cgi );
};
if( $@ ) {
	print $cgi->header("text/plain");
	print "Error initializing Bluestem::CGIClient: $@\n";
	exit 0;
}

unless( -e "$bsc->{conf}{template_dir}/admin.html" ) {
	print $cgi->header("text/plain");
	print "Cannot find admin.html in template directory: $bsc->{conf}{template_dir}\n";
	print "Please check the template_dir setting in $bsc->{config_file}\n";
	print "If the value of template_dir is correct, please check file permissions.";
	exit 0;
}

my $server_key = $cgi->param('server_key');
unless( $server_key and $server_key eq $bsc->{conf}{server_key} ) {
	$bsc->print_template("admin.html",{
		login => 1,
		title => "Bluestem Admin Console",
		errmsg => $server_key ? "Invalid server key." : "",
	});
	exit 0;
}

my $status;
if( $cgi->param('action') eq 'update' ) {
	for my $f (qw(
		cache_dir cgipath client_cookiedomain client_cookiename
		default_auth default_domain id_server_bluestem_path 
		idle_timeout imgpath logfile loglvl
		prior_timeout servername
	)) {
		$bsc->{conf}{$f} = $cgi->param($f) || $bsc->{conf}{$f} || '';
	}
	if( $cgi->param("new_server_key") ) {
		$bsc->{conf}{server_key} = $cgi->param("new_server_key");
		$server_key = $cgi->param("new_server_key");
	}
	if( $cgi->param("new_id_server") ) {
		$bsc->{conf}{id_servers} = {
			$cgi->param("new_id_server") => {primary=>1,weight=>1},
		};
	}
	$bsc->writeconf();
	$status = "Updated Bluestem configuration";
}

my @id_server_list;
for my $ids ( sort keys %{$bsc->{conf}{id_servers}} ) {
	my $conf = $bsc->{conf}{id_servers}{$ids};
	push @id_server_list, { name => $ids, %$conf };
}

my %params = (
	title => "Bluestem Admin Console",
	status => $status,
	server_key => $server_key,
	id_server_list => \@id_server_list,
);

$params{servername_warning} = "";
$params{server_key_warning} =
	( not $bsc->{conf}{server_key} ) ? "No server_key set!" :
	$bsc->{conf}{server_key} !~ m/^[0-9a-fA-F]{64}$/ ? "Server key must consist of characters 0-9 and a-f only." :
	"";
$params{cache_dir_warning} =
	( not $bsc->{conf}{cache_dir} ) ? "No cache directory specified!" :
	( not -e $bsc->{conf}{cache_dir} ) ? "Specified cache directory does not exist!" :
	( not -d $bsc->{conf}{cache_dir} ) ? "Specified path is not a directory!" :
	( not -x $bsc->{conf}{cache_dir} ) ? "Specified directory is not accessible!" :
	( not -w $bsc->{conf}{cache_dir} ) ? "Specified directory is not accessible for writing!" :
	"";
my($cgipath_guess) = $cgi->url('-absolute'=>1) =~ m{(.*)/};
$params{cgipath_warning} = "cgipath should probably be set to $cgipath_guess\n"
	unless $cgipath_guess eq $bsc->{conf}{cgipath} or "$cgipath_guess/" eq $bsc->{conf}{cgipath} or not $cgipath_guess;
$params{logfile_warning} = FileHandle->new($bsc->{conf}{logfile},'>>') ? "" : "Unable to open logfile: $!";

$bsc->print_template("admin.html",\%params);

exit 0;
