#!/usr/bin/perl
#
#   Bluestem - University of Illinois WWW Identification Service
#
#   test.cgi - Bluestem API Test CGI
#

use strict;
use warnings;

use lib "/usr/local/bluestem-client/lib";
use Bluestem::CGIClient ();

my $cgi = CGI->new();
my $bsc = Bluestem::CGIClient->new(
	cgi => $cgi,
);

my $session;
eval {
	$session = $bsc->get_session();
	if( $session ) {
		my @userinfo = ();
		for my $k ( sort keys %{$session->{userinfo}} ) {
			my $v = $session->{userinfo}{$k};
			if( UNIVERSAL::isa($v,'ARRAY') ) {
				$v = join ', ', @$v;
			} elsif( UNIVERSAL::isa($v,'HASH') ) {
				$v = join ', ', map {"$_=$v->{$_}"} keys %$v;
			}
			push @userinfo, { key => $k, value => $v };
		}
		$bsc->print_template("test.html",{
			%{$session->{userinfo}},
			%$session,
			fullid => $bsc->bluestem_id_full( $session->{netid} ),
			shortid => $bsc->bluestem_id_short( $session->{netid} ),
			logout_return_link => $cgi->self_url,
			userinfo => (@userinfo ? \@userinfo : []),
		});
	} elsif( $cgi->param('testlogin') ) {
		$bsc->handle_login(
			"prior"   => ( $cgi->param('priorok') ? 60*60 : 0 ),
			"iserver" => ( $cgi->param('id_server') || undef  ),
		);
	} else {
		my @id_servers;
		for my $ids ( sort keys %{$bsc->{conf}{id_servers}} ) {
			push @id_servers, { idsname=>$ids };
		}
		$bsc->print_template("test.html",{
			show_login_form => 1,
			self_url => $cgi->self_url,
			id_server_options => \@id_servers,
		});
	}
};
if( $@ ) {
	$bsc->print_template("error.html",{
		error => $@,
	});
	exit 0;
}

exit 0;
