#!/usr/bin/perl
use strict;
use warnings;
use lib 'lib';

use FileHandle ();
use DirHandle ();
use Getopt::Long qw(GetOptions);
my($fh,$dh,$content);

my($no_ask);
GetOptions(
	"no-ask" => \$no_ask,
);

my $conf_file = shift @ARGV;

use JSON::PP ();
my $json = JSON::PP->new();
$json->canonical(1);
$json->pretty(1);
$json->relaxed(1);

$| = 1;

sub ask ($;$)
{
	my($q,$d) = @_;
	return $d if $no_ask;

	if( $d ) {
		print "$q [$d] ";
	} else {
		print "$q ";
	}
	my $a = <STDIN>;
	chomp $a;
	return $a || $d;
}

my $conf;
my $conf_update;
if( $conf_file ) {
	$fh = FileHandle->new($conf_file)
		or die "Could not open config file `$conf_file': $!";
	$conf = $json->decode(join('',<$fh>));
	$conf_update = 1;
} else {
	print "Creating new config from template...\n";
	$fh = FileHandle->new("bluestem.conf")
		or die "Could not open config file `bluestem.conf': $!";
	$conf = $json->decode(join('',<$fh>));
}
$conf->{install_dir} = ask("Target installation directory?",$conf->{install_dir});
$conf->{install_dir} =~ s{/$}{};
$conf->{cgi_install_dir} =
	ask("Where should CGI scripts be installed?",$conf->{cgi_install_dir} || "$conf->{install_dir}/cgi");
$conf->{cgipath} =
	ask("What is the webserver's path to the cgi scripts?",$conf->{cgipath});
$conf->{img_install_dir} =
	ask("Where should bluestem images be installed?",$conf->{img_install_dir} || "$conf->{install_dir}/img");
$conf->{imgpath} =
	ask("What is the webserver's path to the bluestem images?",$conf->{imgpath});
$conf->{cache_dir} =
	ask("Bluestem cache directory?",$conf->{cache_dir} || "$conf->{install_dir}/cache");
$conf->{httpd_user} =
	ask("Webserver user?",$conf->{httpd_user});
my $default_servername = $ENV{HOSTNAME} || `/bin/hostname`;
chomp $default_servername;
$conf->{servername} =
	ask("Server hostname?",$conf->{servername} || $default_servername);
$conf->{server_key} =
	ask("Server key:",$conf->{server_key});
my($id_server) = keys %{$conf->{id_servers}};
$id_server = ask("Primary idserver:",$id_server);
$conf->{id_servers} = { $id_server => { primary=>1, weight=>1 } };
$conf_file =
	ask("Where should the config be written?","$conf->{install_dir}/bluestem.conf");

my $install_dir = $conf->{install_dir};
my $cgi_install_dir = $conf->{cgi_install_dir};
my $img_install_dir = $conf->{img_install_dir};
my $httpd_user = $conf->{httpd_user};

for my $k ( keys %$conf ) {
	$conf->{$k} =~ s/INSTALL_DIR/$install_dir/;
	$conf->{$k} =~ s{//+}{/}g;
}

-d $install_dir
	or system "mkdir","-p",$install_dir;

$fh = FileHandle->new($conf_file,">")
	or die "Failed to open $conf_file";
$fh->print( $json->encode($conf),"\n" );
unless( $conf_update ) {
	system "chown", $conf->{httpd_user}, $conf_file;
	chmod 0600, $conf_file;
}

unless( -e "$install_dir/bluestem.httpd.conf" ) {
	$fh = FileHandle->new("bluestem.httpd.conf");
	$content = join('',<$fh>);
	$content =~ s[ /bluestem/img ][ $conf->{imgpath} ]g;
	$content =~ s[ /bluestem/cgi ][ $conf->{cgipath} ]g;
	$content =~ s[ /usr/local/bluestem-client/img ][ $img_install_dir ]g;
	$content =~ s[ /usr/local/bluestem-client/cgi ][ $cgi_install_dir ]g;
	$fh = FileHandle->new("$install_dir/bluestem.httpd.conf",">");
	$fh->print($content);
	$fh->close();
}

-e $conf->{template_dir} or mkdir $conf->{template_dir};
$dh = DirHandle->new("template");
for my $tmpl ( $dh->read() ) {
	next if $tmpl =~ m/^\./;
	system "cp","template/$tmpl","$conf->{template_dir}/$tmpl";
}

system "touch", $conf->{logfile};
chmod 0600, $conf->{logfile};
unless( -d $conf->{cache_dir} ) {
	mkdir $conf->{cache_dir};
	chmod 0700, $conf->{cache_dir};
	system "chown", $httpd_user, $conf->{cache_dir};
}
unless( -e $conf->{logfile} ) {
	system "chown", $httpd_user, $conf->{logfile};
}

-d $img_install_dir or mkdir $img_install_dir;
system "cp", "-rpf", glob("img/*"), $img_install_dir;
-d "$install_dir/lib" or mkdir "$install_dir/lib";
system "cp", "-rpf", glob("lib/*"), "$install_dir/lib";

# Copy over all the cgi scripts, making sure to change the "use lib" statements
-d $cgi_install_dir or mkdir $cgi_install_dir;
$dh = DirHandle->new("cgi");
for my $cgi ( $dh->read() ) {
	next if $cgi =~ m/^\./;
	$fh = FileHandle->new("cgi/$cgi");
	$content = join '', <$fh>;
	$content =~ s{/usr/local/bluestem-client/lib}{$install_dir/lib};
	$fh = FileHandle->new("$cgi_install_dir/$cgi",">")
		or die "Could not write $cgi_install_dir/$cgi: $!";
	$fh->print($content);
	$fh->close();
	chmod 0755, "$cgi_install_dir/$cgi";
}

$fh = FileHandle->new("lib/Bluestem/Client.pm");
$content = join '', <$fh>;
$content =~ s{/usr/local/bluestem-client/bluestem.conf}{$conf_file};
$fh = FileHandle->new("$install_dir/lib/Bluestem/Client.pm",">");
$fh->print($content);
$fh->close();

$fh = FileHandle->new("startup.pl");
$content = join '', <$fh>;
$content =~ s{/usr/local/bluestem-client/lib}{$install_dir/lib};
$fh = FileHandle->new("$install_dir/startup.pl",">");
$fh->print($content);
$fh->close();
chmod 0755, "$install_dir/startup.pl";

$fh = FileHandle->new("bluestem-client-admin");
$content = join '', <$fh>;
$content =~ s{/usr/local/bluestem-client/lib}{$install_dir/lib};
$fh = FileHandle->new("/usr/local/bin/bluestem-client-admin",">");
$fh->print($content);
$fh->close();
chmod 0755, "/usr/local/bin/bluestem-client-admin";

print "\nAn administrative interface can be found with your cgi scripts here:\n\n";
print "  https://$conf->{servername}$conf->{cgipath}/admin.cgi\n\n";
print "Or configuration can be updated manually in\n\n";
print "  $conf_file\n\n";

